%% Clear Workspace
clearvars; close all; clc;

%% Import OpenSim Libraries into Matlab
import org.opensim.modeling.*

%% Import model.
modelName = 'FourBarTest.osim';
analysis_name = 'fourBarP2P';
dirResults = '';
model = Model(modelName);
model.initSystem();
dateLabel = datestr(datetime('now'),'yyyy-mm-dd-HH-MM-SS');

%% Build Study and Problem
study = MocoStudy();
study.setName(analysis_name);

problem = study.updProblem();

modelProcessor = ModelProcessor(model);
problem.setModelProcessor(modelProcessor);

model = modelProcessor.process();

%% Goal
% Move the angle of link C from start to end position in fixed time.
% Objective: Minimize effort.
CO_rz_0 = deg2rad(10); % START
CO_rz_1 = deg2rad(30); % END

effortGoal = MocoControlGoal('effort', 10);
effortGoal.setExponent(3);
problem.addGoal(effortGoal);

%% Bounds
problem.setStateInfo('/jointCO/CO_rz/value', [deg2rad(-180), deg2rad(180)], CO_rz_0, CO_rz_1);
problem.setTimeBounds(0.0, 1.0);
problem.setControlInfo('/motorOA',1e2*[-1, 1]);

%% Configure Solver

solver = study.initCasADiSolver();
% solver = study.initTropterSolver();
solver.set_num_mesh_intervals(50);
solver.set_verbosity(2);
solver.set_optim_solver('ipopt');
solver.set_optim_convergence_tolerance(1e-5);
solver.set_optim_constraint_tolerance(1e-5);
solver.set_optim_max_iterations(1500);
guess = solver.createGuess();
guess.randomizeAdd();
solver.setGuess(guess);

%% Solve
sol = study.solve();

%% Save Motion
fn = [analysis_name dateLabel '.sto'];
sol.write([dirResults fn])
