%% Clear Workspace
clear all; close all; clc;

%% Import OpenSim Libraries into Matlab
import org.opensim.modeling.*

%% Instantiate an (empty) OpenSim Model
model = Model();
model.setName('FourBarTest');

% Get a reference to the ground object
ground = model.getGround();

% Define the acceleration of gravity
model.setGravity(Vec3(0, -9.81, 0));

%% Design parameters.
L_O = 1;
L_A = 1;
L_B = 1.1;
L_C = 1;
t_O = deg2rad(45);
width = 0.05;
depth = 0.05;

%% Build Links
% linkO = buildLink('linkO', L_O, width, depth, 1, model);
linkA = buildLink('linkA', L_A, width, depth, 1, model);
linkB = buildLink('linkB', L_B, width, depth, 1, model);
linkC = buildLink('linkC', L_C, width, depth, 1, model);

%% Build Joints
% Linkage Joints
[jointOA, OA_rz] = buildPinJoint(ground, linkA, L_O, L_A, 'jointOA', 'OA_rz', model);
[jointAB, AB_rz] = buildPinJoint(linkA, linkB, L_A, L_B, 'jointAB', 'AB_rz', model);
[jointBC, BC_rz] = buildPinJoint(linkB, linkC, L_B, L_C, 'jointBC', 'BC_rz', model);
[jointCO, CO_rz] = buildPinJoint(linkC, ground, L_C, L_O, 'jointCO', 'CO_rz', model);


% % Weld O to ground.
% locationInParent    = Vec3(0,0,0);
% orientationInParent = Vec3(0,0,t_O);
% locationInChild     = Vec3(0,0,0);
% orientationInChild  = Vec3(0,0,0);
% jointOG = WeldJoint('jointOG', linkO, locationInParent,...
%     orientationInParent, ground, locationInChild, orientationInChild);
% model.addJoint(jointOG);

%% Build the Motor
bodyA = ground;
bodyB = linkA;
axis = Vec3(0, 0, 01); axisInGround = false;
motorOA = TorqueActuator(bodyA, bodyB, axis, axisInGround);
motorOA.setName('motorOA');
model.addComponent(motorOA);

%% Initialize the System (checks model consistency).
model.initSystem();

% Save the model to a file
model.print('FourBarTest.osim');

%% Local Functions
function link = buildLink(name, L, W, D, m, model)

import org.opensim.modeling.*

link = Body();
link.setName(name);
link.setMass(m);
link.setInertia( Inertia(1,1,1,0,0,0) );

linkGeo = Brick(Vec3(L/2, W/2, D/2));

link.attachGeometry(linkGeo);

model.addComponent(link);

end

function [joint, coord] = buildPinJoint(parentBody, childBody, L_parent, L_child, joint_name, coord_name, model)

import org.opensim.modeling.*

% Make and add a Pin joint for the parent to child
locationInParent    = Vec3(-L_parent/2,0,0);
orientationInParent = Vec3(0,0,0);
locationInChild     = Vec3(L_child/2,0,0);
orientationInChild  = Vec3(0,0,0);

joint = PinJoint(joint_name, parentBody, locationInParent,...
    orientationInParent, childBody, locationInChild, orientationInChild);


% Update the coordinates of the new joint
coord = joint.updCoordinate();
coord.setRange([deg2rad(-360), deg2rad(360)]);
coord.setName(coord_name);
coord.setDefaultValue(0);
coord.setDefaultSpeedValue(0);

% Add Joint to the Model
model.addComponent(joint);

end